using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Content.Pipeline;
using Microsoft.Xna.Framework.Content.Pipeline.Serialization;
using Microsoft.Xna.Framework.Content.Pipeline.Serialization.Compiler;
using Microsoft.Xna.Framework.Content.Pipeline.Processors;
using Microsoft.Xna.Framework.Content.Pipeline.Graphics;

namespace Bespoke.Games.Framework
{
    /// <summary>
    /// 
    /// </summary>
    public class CustomModelData
    {
        /// <summary>
        /// 
        /// </summary>
        public byte[] VertexData;

        /// <summary>
        /// 
        /// </summary>
        public int[] IndexData;
    }

    /// <summary>
    /// 
    /// </summary>
    [ContentProcessor(DisplayName="Bespoke Software - Custom Model Processor")]
    public class CustomModelProcessor : ModelProcessor
    {
        /// <summary>
        /// 
        /// </summary>
        /// <param name="input"></param>
        /// <param name="context"></param>
        /// <returns></returns>
        public override ModelContent Process(NodeContent input, ContentProcessorContext context)
        {
            ModelContent modelContent = base.Process(input, context);

             foreach (ModelMeshContent meshContent in modelContent.Meshes)
             {
                 CustomModelData customData = new CustomModelData();
                 customData.VertexData = meshContent.VertexBuffer.VertexData;
                 customData.IndexData = new int[meshContent.IndexBuffer.Count];
                 for (int i = 0; i < meshContent.IndexBuffer.Count; i++)
                 {
                     customData.IndexData[i] = meshContent.IndexBuffer[i];
                 }

                 meshContent.Tag = customData;
             }

             return modelContent;
        }
    }

    /// <summary>
    /// 
    /// </summary>
    [ContentTypeWriter]
    public class CustomModelWriter : ContentTypeWriter<CustomModelData>
    {
        /// <summary>
        /// 
        /// </summary>
        /// <param name="targetPlatform"></param>
        /// <returns></returns>
        public override string GetRuntimeType(TargetPlatform targetPlatform)
        {
            return typeof(CustomModelData).AssemblyQualifiedName;    
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="targetPlatform"></param>
        /// <returns></returns>
        public override string GetRuntimeReader(TargetPlatform targetPlatform)
        {
            return typeof(CustomModelReader).AssemblyQualifiedName;
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="output"></param>
        /// <param name="value"></param>
        protected override void Write(ContentWriter output, CustomModelData value)
        {
            output.Write(value.VertexData.Length);
            output.Write(value.VertexData);

            output.Write(value.IndexData.Length);
            foreach (int index in value.IndexData)
            {
                output.Write(index);
            }
        }
    }

    /// <summary>
    /// 
    /// </summary>
    public class CustomModelReader : ContentTypeReader<CustomModelData>
    {
        /// <summary>
        /// 
        /// </summary>
        /// <param name="input"></param>
        /// <param name="existingInstance"></param>
        /// <returns></returns>
        protected override CustomModelData Read(ContentReader input, CustomModelData existingInstance)
        {
            CustomModelData modelData = new CustomModelData();

            int vertexCount = input.ReadInt32();
            modelData.VertexData = input.ReadBytes(vertexCount);

            int indexCount = input.ReadInt32();
            modelData.IndexData = new int[indexCount];
            for (int index = 0; index < indexCount; index++)
            {
                modelData.IndexData[index] = input.ReadInt32();
            }

            return modelData;
        }
    }
}
