using System;
using Microsoft.Xna.Framework.Graphics;

namespace Bespoke.Games.Framework
{
    /// <summary>
    /// Stores a color as a set of floats (and is a reference type instead of a value type for less boxing).
    /// </summary>
    public class FloatColor
    {
        /// <summary>
        /// 
        /// </summary>
        public static FloatColor White
        {
            get
            {
                return sWhite;
            }
        }

        /// <summary>
        /// 
        /// </summary>
        public static FloatColor Black
        {
            get
            {
                return sBlack;
            }
        }

        /// <summary>
        /// 
        /// </summary>
        public float R
        {
            get
            {
                return mR;
            }
            set
            {
                mR = value;
            }
        }

        /// <summary>
        /// 
        /// </summary>
        public float G
        {
            get
            {
                return mG;
            }
            set
            {
                mG = value;
            }
        }

        /// <summary>
        /// 
        /// </summary>
        public float B
        {
            get
            {
                return mB;
            }
            set
            {
                mB = value;
            }
        }

        public uint PackedValue
        {
            get
            {
                uint r = ((uint)mR << 0x10);
                uint g = ((uint)mG << 8);
                uint b = (uint)mB;
                uint a = (uint)0xff << 0x18;                

                return (((r | g) | b) | a);
            }
        }

        /// <summary>
        /// Static constructor.
        /// </summary>
        static FloatColor()
        {
            sWhite = new FloatColor(Color.White);
            sBlack = new FloatColor(Color.Black);
        }

        /// <summary>
        /// Default constructor.
        /// </summary>
        public FloatColor()
        {
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="color"></param>
        public FloatColor(FloatColor color)
            : this(color.R, color.G, color.B)
        {
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="color"></param>
        public FloatColor(Color color)
            : this(color.R, color.G, color.B)
        {
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="r"></param>
        /// <param name="g"></param>
        /// <param name="b"></param>
        public FloatColor(float r, float g, float b)
        {
            mR = r;
            mG = g;
            mB = b;
        }
        /// <summary>
        /// 
        /// </summary>
        /// <param name="obj"></param>
        /// <returns></returns>
        public override bool Equals(object obj)
        {
            FloatColor floatColor = obj as FloatColor;
            if (floatColor == null)
            {
                return false;
            }

            return this == floatColor;
        }

        /// <summary>
        /// 
        /// </summary>
        /// <returns></returns>
        public override int GetHashCode()
        {
            return base.GetHashCode();
        }

        /// <summary>
        /// Casting between FloatColor and Color.
        /// </summary>
        /// <param name="floatColor"></param>
        /// <returns></returns>
        public static implicit operator Color(FloatColor floatColor)
        {
            return new Color((byte)floatColor.R, (byte)floatColor.G, (byte)floatColor.B);
        }

        /// <summary>
        /// Casting between Color and FloatColor.
        /// </summary>
        /// <param name="color"></param>
        /// <returns></returns>
        public static implicit operator FloatColor(Color color)
        {
            return new FloatColor(color.R, color.G, color.B);
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="lhs"></param>
        /// <param name="rhs"></param>
        /// <returns></returns>
        public static bool operator ==(FloatColor lhs, FloatColor rhs)
        {
            if (Object.ReferenceEquals(lhs, rhs))
            {
                return true;
            }

            if ((object)lhs == null || (object)rhs == null)
            {
                return false;
            }

            return (lhs.mR == rhs.mR) && (lhs.mG == rhs.mG) && (lhs.mB == rhs.mB);
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="lhs"></param>
        /// <param name="rhs"></param>
        /// <returns></returns>
        public static bool operator !=(FloatColor lhs, FloatColor rhs)
        {
            return !(lhs == rhs);
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="lhs"></param>
        /// <param name="rhs"></param>
        /// <returns></returns>
        public static FloatColor operator +(FloatColor lhs, FloatColor rhs)
        {
            return new FloatColor(lhs.R + rhs.R, lhs.G + rhs.G, lhs.B + rhs.B);
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="lhs"></param>
        /// <param name="rhs"></param>
        /// <returns></returns>
        public static FloatColor operator +(FloatColor lhs, float rhs)
        {
            return new FloatColor(lhs.R + rhs, lhs.G + rhs, lhs.B + rhs);
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="lhs"></param>
        /// <param name="rhs"></param>
        /// <returns></returns>
        public static FloatColor operator *(FloatColor lhs, FloatColor rhs)
        {
            return new FloatColor(lhs.R * rhs.R, lhs.G * rhs.G, lhs.B * rhs.B);
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="lhs"></param>
        /// <param name="rhs"></param>
        /// <returns></returns>
        public static FloatColor operator *(FloatColor lhs, float rhs)
        {
            return new FloatColor(lhs.R * rhs, lhs.G * rhs, lhs.B * rhs);
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="lhs"></param>
        /// <param name="rhs"></param>
        /// <returns></returns>
        public static FloatColor operator /(FloatColor lhs, float rhs)
        {
            float rhsInverse = 1 / rhs;
            return new FloatColor(lhs.R * rhsInverse, lhs.G * rhsInverse, lhs.B * rhsInverse);
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="color"></param>
        /// <param name="factor"></param>
        /// <param name="maximum"></param>
        /// <returns></returns>
        public static FloatColor Normalize(FloatColor color, float factor, float maximum)
        {
            color.R *= factor;
            if (color.R > maximum)
            {
                color.R = maximum;
            }

            color.G *= factor;
            if (color.G > maximum)
            {
                color.G = maximum;
            }

            color.B *= factor;
            if (color.B > maximum)
            {
                color.B = maximum;
            }

            return color;
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="factor"></param>
        /// <param name="maximum"></param>
        /// <returns></returns>
        public FloatColor Normalize(float factor, float maximum)
        {
            return Normalize(this, factor, maximum);
        }

        /// <summary>
        /// 
        /// </summary>
        /// <returns></returns>
        public override string ToString()
        {
            return string.Format("R:{0}, G:{1}, B:{2}", mR, mG, mB);
        }

        private static readonly FloatColor sWhite;
        private static readonly FloatColor sBlack;

        private float mR;
        private float mG;
        private float mB;
    }
}
