using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;

namespace Bespoke.Games.Framework
{
    /// <summary>
    /// 
    /// </summary>
    public partial class FpsComponent : Microsoft.Xna.Framework.DrawableGameComponent
    {
        /// <summary>
        /// 
        /// </summary>
        public enum ScreenLocation
        {
            /// <summary>
            /// 
            /// </summary>
            TitleBar,

            /// <summary>
            /// 
            /// </summary>
            UpperLeft,

            /// <summary>
            /// 
            /// </summary>
            UpperRight,

            /// <summary>
            /// 
            /// </summary>
            LowerLeft,

            /// <summary>
            /// 
            /// </summary>
            LowerRight
        }

        #region Properties

        /// <summary>
        /// 
        /// </summary>
        public SpriteFont Font
        {
            get
            {
                return mFont;
            }
            set
            {
                mFont = value;
            }
        }

        /// <summary>
        /// 
        /// </summary>
        public ScreenLocation Location
        {
            get
            {
                return mLocation;
            }
            set
            {
                mLocation = value;
            }
        }

        /// <summary>
        /// 
        /// </summary>
        public Color Color
        {
            get
            {
                return mColor;
            }
            set
            {
                mColor = value;
            }
        }

        /// <summary>
        /// 
        /// </summary>
        public int CurrentFrameRate
        {
            get
            {
                return mCurrentFrameRate;
            }
        }

        /// <summary>
        /// 
        /// </summary>
        public bool IsFixedTimeStep
        {
            get
            {
                return Game.IsFixedTimeStep;
            }
            set
            {
                Game.IsFixedTimeStep = value;
            }
        }

        /// <summary>
        /// 
        /// </summary>
        public int TargetFrameRate
        {
            get
            {
                return mTargetFrameRate;
            }
            set
            {
                if (mTargetFrameRate != value)
                {
                    if (value < 1)
                    {
                        mTargetFrameRate = 1;
                    }
                    else if (value > 1000)
                    {
                        mTargetFrameRate = 1000;
                    }
                    else
                    {
                        mTargetFrameRate = value;
                    }

                    UpdateTargetElapsedTime();
                }
            }
        }

        #endregion

        /// <summary>
        /// 
        /// </summary>
        /// <param name="game"></param>
        public FpsComponent(Game game)
            : base(game)
        {
            mLocation = ScreenLocation.UpperLeft;
            mColor = Color.White;
            mTargetFrameRate = 60;
        }

        /// <summary>
        /// Allows the game component to perform any initialization it needs to before starting
        /// to run.  This is where it can query for any required services and load content.
        /// </summary>
        public override void Initialize()
        {
            mRightX = Game.Window.ClientBounds.Width - 50;
            mLowerY = Game.Window.ClientBounds.Height - SCREEN_DRAW_OFFSET;
            mLastTickCount = Environment.TickCount;
            UpdateTargetElapsedTime();

            base.Initialize();
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="loadAllContent"></param>
        protected override void LoadGraphicsContent(bool loadAllContent)
        {            
            if (loadAllContent)
            {
                mContentManager = new ContentManager(Game.Services);
                mFont = mContentManager.Load<SpriteFont>(@"Content\Fonts\Arial_12_Regular");
            }   
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="unloadAllContent"></param>
        protected override void UnloadGraphicsContent(bool unloadAllContent)
        {
            if (unloadAllContent)
            {
                mContentManager.Unload();
            }
        }

        /// <summary>
        /// Allows the game component to update itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        public override void Update(GameTime gameTime)
        {
            if ((Environment.TickCount - mLastTickCount) >= 1000)
            {
                mLastTickCount = Environment.TickCount;
                mCurrentFrameRate = mFrameCount;
                mFrameCount = 0;
            }

            mFrameCount++;

            base.Update(gameTime);
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="gameTime"></param>
        public override void Draw(GameTime gameTime)
        {
            string fpsString = String.Format(" [{0} FPS]", mCurrentFrameRate);
            switch (mLocation)
                {
                case ScreenLocation.TitleBar:
                    string titleString = Game.Window.Title;
                    int index = titleString.IndexOf(" [");
                    if (index != -1)
                    {
                        titleString = titleString.Substring(0, index);
                    }
                    Game.Window.Title = titleString + fpsString;
                    break;

                case ScreenLocation.UpperLeft:
                    SpriteManager.DrawString(mFont, fpsString, SCREEN_DRAW_OFFSET, SCREEN_DRAW_OFFSET, mColor);
                    break;

                case ScreenLocation.UpperRight:
                    SpriteManager.DrawString(mFont, fpsString, mRightX, SCREEN_DRAW_OFFSET, mColor);
                    break;

                case ScreenLocation.LowerLeft:
                    SpriteManager.DrawString(mFont, fpsString, SCREEN_DRAW_OFFSET, mLowerY, mColor);
                    break;

                case ScreenLocation.LowerRight:
                    SpriteManager.DrawString(mFont, fpsString, mRightX, mLowerY, mColor);
                    break;
                }

            base.Draw(gameTime);
        }

        /// <summary>
        /// 
        /// </summary>
        private void UpdateTargetElapsedTime()
        {
            Game.TargetElapsedTime = new TimeSpan(0, 0, 0, 0, 1000 / mTargetFrameRate);
        }

        private const int SCREEN_DRAW_OFFSET = 10;
        private int mRightX;
        private int mLowerY;

        private ContentManager mContentManager;
        private SpriteFont mFont;
        private ScreenLocation mLocation;
        private Color mColor;
        private int mLastTickCount;
        private int mFrameCount;
        private int mCurrentFrameRate;
        private int mTargetFrameRate;
    }
}