using System;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Input;

namespace Bespoke.Games.Framework
{
    /// <summary>
    ///
    /// </summary>
    public class GamePadComponent : Microsoft.Xna.Framework.DrawableGameComponent
    {
        #region Events

        /// <summary>
        /// 
        /// </summary>
        public event CustomDrawHandler CustomDraw;

        #endregion

        #region Properties

        /// <summary>
        /// 
        /// </summary>
        public PlayerIndex PlayerIndex
        {
            get
            {
                return mPlayerIndex;
            }
        }

        /// <summary>
        /// 
        /// </summary>
        public GamePadState CurrentState
        {
            get
            {
                return mCurrentState;
            }
        }

        /// <summary>
        /// 
        /// </summary>
        public GamePadState LastState
        {
            get
            {
                return mLastState;
            }
        }

        /// <summary>
        /// 
        /// </summary>
        public bool IsConnected
        {
            get
            {
                return mCurrentState.IsConnected;
            }
        }

        #endregion

        /// <summary>
        /// 
        /// </summary>
        /// <param name="game"></param>
        /// <param name="playerIndex"></param>
        public GamePadComponent(Game game, PlayerIndex playerIndex)
            : base(game)
        {
            mPlayerIndex = playerIndex;
        }

        /// <summary>
        /// Allows the game component to perform any initialization it needs to before starting
        /// to run.  This is where it can query for any required services and load content.
        /// </summary>
        public override void Initialize()
        {
            mCurrentState = GamePad.GetState(mPlayerIndex);
            mLastState = mCurrentState;

            base.Initialize();
        }

        /// <summary>
        /// Allows the game component to update itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        public override void Update(GameTime gameTime)
        {
            mLastState = mCurrentState;
            mCurrentState = GamePad.GetState(mPlayerIndex);

            base.Update(gameTime);
        }

        /// <summary>
        /// Allows the game component to draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        public override void Draw(GameTime gameTime)
        {
            if (CustomDraw != null)
            {
                CustomDraw(this, new CustomDrawEventArgs(gameTime));
            }

            base.Draw(gameTime);
        }

        private PlayerIndex mPlayerIndex;
        private GamePadState mCurrentState;
        private GamePadState mLastState;        
    }
}