using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Input;

namespace Bespoke.Games.Framework
{
    /// <summary>
    /// 
    /// </summary>
    public class KeyboardComponent : Microsoft.Xna.Framework.GameComponent
    {
        #region Properties

        /// <summary>
        /// 
        /// </summary>
        public KeyboardState CurrentState
        {
            get
            {
                return mCurrentState;
            }
        }

        public Keys[] PressedKeys
        {
            get
            {
                return mCurrentState.GetPressedKeys();
            }
        }

        #endregion

        public KeyboardComponent(Game game)
            : base(game)
        {
            mKeysPressedThisFrame = new List<Keys>();
            mKeysReleasedThisFrame = new List<Keys>();
        }

        /// <summary>
        /// Allows the game component to perform any initialization it needs to before starting
        /// to run.  This is where it can query for any required services and load content.
        /// </summary>
        public override void Initialize()
        {
            mCurrentState = Keyboard.GetState();
            mPreviouslyPressedKeys = mCurrentState.GetPressedKeys();

            base.Initialize();
        }

        /// <summary>
        /// Allows the game component to update itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        public override void Update(GameTime gameTime)
        {
            mKeysPressedThisFrame.Clear();
            mKeysReleasedThisFrame.Clear();

            mCurrentState = Keyboard.GetState();
            mNewlyPressedKeys = mCurrentState.GetPressedKeys();

            foreach (Keys newlyPressedKey in mNewlyPressedKeys)
            {
                bool found = false;
                foreach (Keys previouslyPressedKey in mPreviouslyPressedKeys)
                {
                    if (newlyPressedKey == previouslyPressedKey)
                    {
                        found = true;
                        break;
                    }
                }

                if (found == false)
                {
                    mKeysPressedThisFrame.Add(newlyPressedKey);
                }
            }

            foreach (Keys previouslyPressedKey in mPreviouslyPressedKeys)
            {
                bool found = false;
                foreach (Keys newlyPressedKey in mNewlyPressedKeys)
                {
                    if (previouslyPressedKey == newlyPressedKey)
                    {
                        found = true;
                        break;
                    }
                }

                if (found == false)
                {
                    mKeysReleasedThisFrame.Add(previouslyPressedKey);
                }
            }

            mPreviouslyPressedKeys = mNewlyPressedKeys;

            base.Update(gameTime);
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="key"></param>
        /// <returns></returns>
        public bool IsKeyDown(Keys key)
        {
            return mCurrentState.IsKeyDown(key);
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="key"></param>
        /// <returns></returns>
        public bool IsKeyUp(Keys key)
        {
            return mCurrentState.IsKeyUp(key);
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="key"></param>
        /// <returns></returns>
        public bool WasKeyPressedThisFrame(Keys key)
        {
            return mKeysPressedThisFrame.Contains(key);
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="key"></param>
        /// <returns></returns>
        public bool WasKeyReleasedThisFrame(Keys key)
        {
            return mKeysReleasedThisFrame.Contains(key);
        }

        private KeyboardState mCurrentState;
        private Keys[] mPreviouslyPressedKeys;
        private Keys[] mNewlyPressedKeys;
        private List<Keys> mKeysPressedThisFrame;
        private List<Keys> mKeysReleasedThisFrame;
    }
}


