using System;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Input;

namespace Bespoke.Games.Framework
{
    /// <summary>
    ///
    /// </summary>
    public class MouseComponent : Microsoft.Xna.Framework.DrawableGameComponent
    {
        #region Events

        /// <summary>
        /// 
        /// </summary>
        public event CustomDrawHandler CustomDraw;

        #endregion

        #region Properties

        /// <summary>
        /// 
        /// </summary>
        public MouseState CurrentState
        {
            get
            {
                return mCurrentState;
            }
        }

        /// <summary>
        /// 
        /// </summary>
        public MouseState LastState
        {
            get
            {
                return mLastState;
            }
        }

        /// <summary>
        /// 
        /// </summary>
        public Vector2 AbsolutePosition
        {
            get
            {
                return new Vector2(mCurrentState.X, mCurrentState.Y);
            }
        }

        /// <summary>
        /// 
        /// </summary>
        public int AbsolutePositionX
        {
            get
            {
                return mCurrentState.X;
            }
        }

        /// <summary>
        /// 
        /// </summary>
        public int AbsolutePositionY
        {
            get
            {
                return mCurrentState.Y;
            }
        }

        /// <summary>
        /// 
        /// </summary>
        public int AbsoluteScrollWheelValue
        {
            get
            {
                return mCurrentState.ScrollWheelValue;
            }
        }

        /// <summary>
        /// 
        /// </summary>
        public Vector2 RelativePosition
        {
            get
            {
                return new Vector2(mCurrentState.X - mLastState.X, mCurrentState.Y - mLastState.Y);
            }
        }

        /// <summary>
        /// 
        /// </summary>
        public int RelativePositionX
        {
            get
            {
                return mCurrentState.X - mLastState.X;
            }
        }

        /// <summary>
        /// 
        /// </summary>
        public int RelativePositionY
        {
            get
            {
                return mCurrentState.Y - mLastState.Y;
            }
        }

        /// <summary>
        /// 
        /// </summary>
        public int RelativeScrollWheelValue
        {
            get
            {
                return mCurrentState.ScrollWheelValue - mLastState.ScrollWheelValue;
            }
        }

        /// <summary>
        /// 
        /// </summary>
        /// <returns></returns>
        public bool IsLeftButtonDown
        {
            get
            {
                return mCurrentState.LeftButton == ButtonState.Pressed;
            }
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="ignoreRepeats"></param>
        /// <returns></returns>
        public bool IsLeftButtonHeldDown
        {
            get
            {
                return IsLeftButtonDown && WasLeftButtonDown;
            }
        }

        /// <summary>
        /// 
        /// </summary>
        /// <returns></returns>
        public bool WasLeftButtonDown
        {
            get
            {
                return mLastState.LeftButton == ButtonState.Pressed;
            }
        }

        /// <summary>
        /// 
        /// </summary>
        /// <returns></returns>
        public bool IsRightButtonDown
        {
            get
            {
                return mCurrentState.RightButton == ButtonState.Pressed;
            }
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="ignoreRepeats"></param>
        /// <returns></returns>
        public bool IsRightButtonHeldDown
        {
            get
            {
                return IsRightButtonDown && WasRightButtonDown;
            }
        }

        /// <summary>
        /// 
        /// </summary>
        /// <returns></returns>
        public bool WasRightButtonDown
        {
            get
            {
                return mLastState.RightButton == ButtonState.Pressed;
            }
        }

        /// <summary>
        /// 
        /// </summary>
        /// <returns></returns>
        public bool IsMiddleButtonDown
        {
            get
            {
                return mCurrentState.MiddleButton == ButtonState.Pressed;
            }
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="ignoreRepeats"></param>
        /// <returns></returns>
        public bool IsMiddleButtonHeldDown
        {
            get
            {
                return IsMiddleButtonDown && WasMiddleButtonDown;
            }
        }

        /// <summary>
        /// 
        /// </summary>
        /// <returns></returns>
        public bool WasMiddleButtonDown
        {
            get
            {
                return mLastState.MiddleButton == ButtonState.Pressed;
            }
        }

        /// <summary>
        /// 
        /// </summary>
        /// <returns></returns>
        public bool IsXButton1Down
        {
            get
            {
                return mCurrentState.XButton1 == ButtonState.Pressed;
            }
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="ignoreRepeats"></param>
        /// <returns></returns>
        public bool IsXButton1HeldDown
        {
            get
            {
                return IsXButton1Down && WasXButton1Down;
            }
        }

        /// <summary>
        /// 
        /// </summary>
        /// <returns></returns>
        public bool WasXButton1Down
        {
            get
            {
                return mLastState.XButton1 == ButtonState.Pressed;
            }
        }

        /// <summary>
        /// 
        /// </summary>
        /// <returns></returns>
        public bool IsXButton2Down
        {
            get
            {
                return mCurrentState.XButton2 == ButtonState.Pressed;
            }
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="ignoreRepeats"></param>
        /// <returns></returns>
        public bool IsXButton2HeldDown
        {
            get
            {
                return IsXButton2Down && WasXButton2Down;
            }
        }

        /// <summary>
        /// 
        /// </summary>
        /// <returns></returns>
        public bool WasXButton2Down
        {
            get
            {
                return mLastState.XButton2 == ButtonState.Pressed;
            }
        }

        #endregion

        /// <summary>
        /// 
        /// </summary>
        /// <param name="game"></param>
        public MouseComponent(Game game)
            : base(game)
        {
        }

        /// <summary>
        /// Allows the game component to perform any initialization it needs to before starting
        /// to run.  This is where it can query for any required services and load content.
        /// </summary>
        public override void Initialize()
        {
            Mouse.WindowHandle = Game.Window.Handle;
            mCurrentState = Mouse.GetState();
            mLastState = mCurrentState;

            base.Initialize();
        }

        /// <summary>
        /// Allows the game component to update itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        public override void Update(GameTime gameTime)
        {
            mLastState = mCurrentState;
            mCurrentState = Mouse.GetState();

            base.Update(gameTime);
        }

        /// <summary>
        /// Allows the game component to draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        public override void Draw(GameTime gameTime)
        {
            if (CustomDraw != null)
            {
                CustomDraw(this, new CustomDrawEventArgs(gameTime));
            }  

            base.Draw(gameTime);
        }

        private MouseState mCurrentState;
        private MouseState mLastState;
    }
}


