using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;

namespace Bespoke.Games.Framework
{
    public static class SceneManager
    {
        /// <summary>
        /// 
        /// </summary>
        static SceneManager()
        {
            sActors = new List<Actor>();
        }

        /// <summary>
        /// 
        /// </summary>
        public static void Clear()
        {
            sActors.Clear();
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="actor"></param>
        public static void AddActor(Actor actor)
        {
            sActors.Add(actor);
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="actor"></param>
        public static void RemoveActor(Actor actor)
        {
            sActors.Remove(actor);
        }

        /// <summary>
        /// Initialize all actors in the scene.
        /// </summary>
        public static void Initialize()
        {
            if (sActors.Count > 0)
            {
                foreach (Actor actor in sActors)
                {
                    actor.Initialize();
                }
            }
        }

        /// <summary>
        /// Update all actors in the scene.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        public static void Update(GameTime gameTime)
        {
            if (sActors.Count > 0)
            {
                foreach (Actor actor in sActors)
                {
                    actor.Update(gameTime);
                }
            }
        }

        /// <summary>
        /// Draw all actors in the scene.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        public static void Draw(GameTime gameTime)
        {
            if (sActors.Count > 0)
            {
                foreach (Actor actor in sActors)
                {
                    actor.Draw(gameTime);
                }
            }
        }

        private static List<Actor> sActors;
    }
}