using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Content;

namespace Bespoke.Games.Framework
{
    /// <summary>
    /// 
    /// </summary>
    public static class SpriteManager
    {
        /// <summary>
        /// 
        /// </summary>
        public static SpriteFont DefaultFont
        {
            get
            {
                return sDefaultFont;
            }
            set
            {
                sDefaultFont = value;
            }
        }

        /// <summary>
        /// 
        /// </summary>
        static SpriteManager()
        {
            sSprites = new List<Sprite>();
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="game"></param>
        public static void Init(Game game)
        {
            ContentManager contentManager = new ContentManager(game.Services);
            sDefaultFont = contentManager.Load<SpriteFont>(@"Content\Fonts\Arial_12_Regular");

            IGraphicsDeviceService graphicsService = (IGraphicsDeviceService)game.Services.GetService(typeof(IGraphicsDeviceService));
            sSpriteBatch = new SpriteBatch(graphicsService.GraphicsDevice);            
            sSprites.Clear();
        }

        /// <summary>
        /// 
        /// </summary>
        public static void Clear()
        {
            sSprites.Clear();
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="sprite"></param>
        public static void AddSprite(Sprite sprite)
        {
            sSprites.Add(sprite);
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="sprite"></param>
        public static void RemoveSprite(Sprite sprite)
        {
            sSprites.Remove(sprite);
        }

        /// <summary>
        /// 
        /// </summary>
        public static void Draw()
        {
            if (sSprites.Count > 0)
            {
                sSpriteBatch.Begin(SpriteBlendMode.AlphaBlend, SpriteSortMode.Deferred, SaveStateMode.SaveState);

                foreach (Sprite sprite in sSprites)
                {
                    sSpriteBatch.Draw(sprite.Texture, sprite.Rectangle, Color.White);
                }

                sSpriteBatch.End();
            }
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="x"></param>
        /// <param name="y"></param>
        /// <param name="color"></param>
        public static void DrawString(string text, float x, float y, Color color)
        {
            DrawString(sDefaultFont, text, x, y, color);
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="font"></param>
        /// <param name="x"></param>
        /// <param name="y"></param>
        /// <param name="color"></param>
        public static void DrawString(SpriteFont font, string text, float x, float y, Color color)
        {
            sSpriteBatch.Begin(SpriteBlendMode.AlphaBlend, SpriteSortMode.Deferred, SaveStateMode.SaveState);
            sSpriteBatch.DrawString(font, text, new Vector2(x, y), color);
            sSpriteBatch.End();
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="x"></param>
        /// <param name="y"></param>
        /// <param name="color"></param>
        public static void DrawString(string text, Vector2 position, Color color)
        {
            DrawString(sDefaultFont, text, position, color);
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="font"></param>
        /// <param name="text"></param>
        /// <param name="position"></param>
        /// <param name="color"></param>
        public static void DrawString(SpriteFont font, string text, Vector2 position, Color color)
        {
            sSpriteBatch.Begin(SpriteBlendMode.AlphaBlend, SpriteSortMode.Deferred, SaveStateMode.SaveState);
            sSpriteBatch.DrawString(font, text, position, color);
            sSpriteBatch.End();
        }

        public static void DrawString(SpriteFont font, string text, Vector2 position, Color color, float rotation,
            Vector2 origin, float scale, SpriteEffects effects, float layerDepth)
        {
            sSpriteBatch.Begin(SpriteBlendMode.AlphaBlend, SpriteSortMode.Deferred, SaveStateMode.SaveState);
            sSpriteBatch.DrawString(font, text, position, color, rotation, origin, scale, effects, layerDepth);
            sSpriteBatch.End();
        }

        private static SpriteFont sDefaultFont;
        private static SpriteBatch sSpriteBatch;
        private static List<Sprite> sSprites;
    }
}
