using System;
using System.Collections.ObjectModel;
using System.Collections.Generic;
using ColorMatching.Content;

namespace ColorMatching
{
    /// <summary>
    /// 
    /// </summary>
    public class CIEColorSpaceEntry
    {
        /// <summary>
        /// 
        /// </summary>
        public float WaveLength
        {
            get
            {
                return mWaveLength;
            }
        }

        /// <summary>
        /// 
        /// </summary>
        public float X
        {
            get
            {
                return mX;
            }
        }

        /// <summary>
        /// 
        /// </summary>
        public float Y
        {
            get
            {
                return mY;
            }
        }

        /// <summary>
        /// 
        /// </summary>
        public float Z
        {
            get
            {
                return mZ;
            }
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="waveLength"></param>
        /// <param name="x"></param>
        /// <param name="y"></param>
        /// <param name="z"></param>
        public CIEColorSpaceEntry(float waveLength, float x, float y, float z)
        {
            mWaveLength = waveLength;
            mX = x;
            mY = y;
            mZ = z;
        }

        private float mWaveLength;
        private float mX;
        private float mY;
        private float mZ;
    }

    /// <summary>
    /// 
    /// </summary>
    public static class CIEColorSpace
    {
        /// <summary>
        /// 
        /// </summary>
        public static IList<CIEColorSpaceEntry> Entries
        {
            get
            {
                return mEntries.Values;
            }
        }

        /// <summary>
        /// 
        /// </summary>
        static CIEColorSpace()
        {
            mEntries = new SortedList<float, CIEColorSpaceEntry>();

            string[] stringEntries = Resources.ciexyz64.Split("\n".ToCharArray());
            foreach (string stringEntry in stringEntries)
            {
                if (stringEntry == String.Empty)
                {
                    continue;
                }

                string[] stringValues = stringEntry.Split(",".ToCharArray());
                if (stringValues.Length != 4)
                {
                    throw new Exception("Invalid CIE Color Space Entry.");
                }

                float waveLength = float.Parse(stringValues[0]);
                float x = float.Parse(stringValues[1]);
                float y = float.Parse(stringValues[2]);
                float z = float.Parse(stringValues[3]);

                mEntries.Add(waveLength, new CIEColorSpaceEntry(waveLength, x, y, z));
            }
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="waveLength"></param>
        /// <returns></returns>
        public static CIEColorSpaceEntry GetEntry(float waveLength)
        {
            return mEntries[waveLength];
        }


        /// <summary>
        /// 
        /// </summary>
        /// <param name="spectrum"></param>
        /// <returns></returns>
        public static XYZCoordinate SpectrumToXYZCoordinate(Spectrum spectrum)
        {
            XYZCoordinate xyzCoordinate = new XYZCoordinate();

            foreach (SpectrumSample sample in spectrum)
            {
                CIEColorSpaceEntry cieEntry = mEntries[sample.WaveLength];
                xyzCoordinate.X += sample.Value * cieEntry.X * WaveLengthDelta;
                xyzCoordinate.Y += sample.Value * cieEntry.Y * WaveLengthDelta;
                xyzCoordinate.Z += sample.Value * cieEntry.Z * WaveLengthDelta;
            }
            
            return xyzCoordinate;
        }

        private static readonly float WaveLengthDelta = 5.0f;

        private static SortedList<float, CIEColorSpaceEntry> mEntries;
    }
}