using System;
using System.Diagnostics;
using System.IO;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Storage;
using Bespoke.Games.Framework;

namespace ColorMatching
{
    /// <summary>
    /// 
    /// </summary>
    public class ColorMatching : Microsoft.Xna.Framework.Game
    {
        /// <summary>
        /// Default constructor.
        /// </summary>
        public ColorMatching()
        {
            mGraphicsDeviceManager = new GraphicsDeviceManager(this);
            mContentManager = new ContentManager(Services);

            mGraphicsDeviceManager.PreferredBackBufferWidth = CLIENT_WIDTH;
            mGraphicsDeviceManager.PreferredBackBufferHeight = CLIENT_HEIGHT;

            mKeyboardComponent = new KeyboardComponent(this);
            Components.Add(mKeyboardComponent);
        }

        /// <summary>
        ///
        /// </summary>
        protected override void Initialize()
        {
            IsMouseVisible = true;

            base.Initialize();
        }

        /// <summary>
        ///
        /// </summary>
        /// <param name="loadAllContent">Which type of content to load.</param>
        protected override void LoadGraphicsContent(bool loadAllContent)
        {
            if (loadAllContent)
            {
                // Convert the Macbeth Color Checker into RGB values.
                List<FloatColor> patchColors = new List<FloatColor>(MacbethColorChecker.Patches.Length);
                foreach (Spectrum spectrum in MacbethColorChecker.Patches)
                {
                    XYZCoordinate xyzCoordinate = CIEColorSpace.SpectrumToXYZCoordinate(spectrum);
                    FloatColor rgb = NtscChromaticity.XYZCoordinateToRGB(xyzCoordinate);
                    patchColors.Add(rgb.Normalize(255, 255));                    
                }

                // Build the textures to display the results
                mSpriteBatch = new SpriteBatch(mGraphicsDeviceManager.GraphicsDevice);

                mPatchWidth = (int)(CLIENT_WIDTH / TextureColumns);
                mPatchHeight = (int)(CLIENT_HEIGHT / TextureRows);
                uint[] colorData = new uint[mPatchWidth * mPatchHeight];

                mPatchTextures = new List<Texture2D>(patchColors.Count);
                foreach (FloatColor patchColor in patchColors)
                {
                    Texture2D patchTexture = new Texture2D(mGraphicsDeviceManager.GraphicsDevice, mPatchWidth, mPatchHeight, 1, ResourceUsage.None, SurfaceFormat.Color, ResourceManagementMode.Automatic);

                    for (int i = 0; i < colorData.Length; i++)
                    {
                        colorData[i] = patchColor.PackedValue;
                    }

                    patchTexture.SetData<uint>(colorData);
                    mPatchTextures.Add(patchTexture);
                }                
            }
        }

        /// <summary>
        /// Unload your graphics content.  If unloadAllContent is true, you should
        /// unload content from both ResourceManagementMode pools.  Otherwise, just
        /// unload ResourceManagementMode.Manual content.  Manual content will get
        /// Disposed by the GraphicsDevice during a Reset.
        /// </summary>
        /// <param name="unloadAllContent">Which type of content to unload.</param>
        protected override void UnloadGraphicsContent(bool unloadAllContent)
        {
            if (unloadAllContent == true)
            {
                mContentManager.Unload();
            }
        }

        /// <summary>
        /// Allows the game to run logic such as updating the world,
        /// checking for collisions, gathering input and playing audio.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Update(GameTime gameTime)
        {
            if (mKeyboardComponent.WasKeyPressedThisFrame(Keys.Escape))
            {
                Exit();
            }

            base.Update(gameTime);
        }

        /// <summary>
        /// This is called when the game should draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Draw(GameTime gameTime)
        {
            mGraphicsDeviceManager.GraphicsDevice.Clear(Color.CornflowerBlue);

            mSpriteBatch.Begin(SpriteBlendMode.None, SpriteSortMode.Deferred, SaveStateMode.SaveState);

            int row = 0;
            int column = 0;
            Vector2 position = Vector2.Zero;
            foreach (Texture2D patchTexture in mPatchTextures)
            {
                position.X = column * mPatchWidth;
                position.Y = row * mPatchHeight;
                mSpriteBatch.Draw(patchTexture, position, Color.White);

                column++;
                if (column == TextureColumns)
                {
                    column = 0;
                    row++;
                }
            }

            mSpriteBatch.End();

            base.Draw(gameTime);
        }

        private static readonly int CLIENT_WIDTH = 636;
        private static readonly int CLIENT_HEIGHT = 480;

        private static readonly int TextureRows = 4;
        private static readonly int TextureColumns = 6;

        private ContentManager mContentManager;
        private GraphicsDeviceManager mGraphicsDeviceManager;
        private KeyboardComponent mKeyboardComponent;
        private SpriteBatch mSpriteBatch;
        private List<Texture2D> mPatchTextures;
        private int mPatchWidth;
        private int mPatchHeight;
    }
}