using System;
using Microsoft.Xna.Framework;
using Bespoke.Games.Framework;

namespace ColorMatching
{
    /// <summary>
    /// 
    /// </summary>
    public class ChromaticityCoordinate
    {
        /// <summary>
        /// 
        /// </summary>
        public FloatColor RGB
        {
            get
            {
                return mRGB;
            }
            set
            {
                mRGB = value;
            }
        }

        /// <summary>
        /// 
        /// </summary>
        public float WhitePoint
        {
            get
            {
                return mWhitePoint;
            }
            set
            {
                mWhitePoint = value;
            }
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="rgb"></param>
        /// <param name="whitePoint"></param>
        public ChromaticityCoordinate(FloatColor rgb, float whitePoint)
        {
            mRGB = rgb;
            mWhitePoint = whitePoint;
        }

        private FloatColor mRGB;
        private float mWhitePoint;
    }

    /// <summary>
    /// 
    /// </summary>
    public static class NtscChromaticity
    {
        /// <summary>
        /// 
        /// </summary>
        public static ChromaticityCoordinate X
        {
            get
            {
                return sX;
            }
        }

        /// <summary>
        /// 
        /// </summary>
        public static ChromaticityCoordinate Y
        {
            get
            {
                return sY;
            }
        }

        /// <summary>
        /// 
        /// </summary>
        public static float Luminance
        {
            get
            {
                return sWhiteXYZCoordinate.Y;
            }
            set
            {
                InitWhiteXYZCoordinate(value);
            }
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="luminance"></param>
        static NtscChromaticity()
        {
            sX = new ChromaticityCoordinate(new FloatColor(0.67f, 0.21f, 0.14f), 0.313f);
            sY = new ChromaticityCoordinate(new FloatColor(0.33f, 0.71f, 0.08f), 0.329f);
            sWhiteXYZCoordinate = new XYZCoordinate();
            sRedXYZCoordinate = new XYZCoordinate();
            sGreenXYZCoordinate = new XYZCoordinate();
            sBlueXYZCoordinate = new XYZCoordinate();
            InitWhiteXYZCoordinate(DefaultLuminance);
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="luminance"></param>
        private static void InitWhiteXYZCoordinate(float luminance)
        {
            sWhiteXYZCoordinate.Y = luminance;
            float yRatio = (sWhiteXYZCoordinate.Y / sY.WhitePoint);

            sWhiteXYZCoordinate.X = sX.WhitePoint * yRatio;
            sWhiteXYZCoordinate.Z = (1 - sX.WhitePoint - sY.WhitePoint) * yRatio;

            Matrix chromaticityMatrix = Matrix.Invert(new Matrix(sX.RGB.R, sX.RGB.G, sX.RGB.B, 0,
                                                                 sY.RGB.R, sY.RGB.G, sY.RGB.B, 0,
                                                                 1 - sX.RGB.R - sY.RGB.R, 1 - sX.RGB.G - sY.RGB.G, 1 - sX.RGB.B - sY.RGB.B, 0,
                                                                 0, 0, 0, 1));

            Matrix xyzMatrix = new Matrix();
            xyzMatrix.M11 = sWhiteXYZCoordinate.X;
            xyzMatrix.M21 = sWhiteXYZCoordinate.Y;
            xyzMatrix.M31 = sWhiteXYZCoordinate.Z;

            Matrix sumMatrix = chromaticityMatrix * xyzMatrix;
            float sumRed = sumMatrix.M11;
            float sumGreen = sumMatrix.M21;
            float sumBlue = sumMatrix.M31;

            sRedXYZCoordinate.X = sX.RGB.R * sumRed;
            sRedXYZCoordinate.Y = sY.RGB.R * sumRed;
            sRedXYZCoordinate.Z = (1 - sX.RGB.R - sY.RGB.R) * sumRed;

            sGreenXYZCoordinate.X = sX.RGB.G * sumGreen;
            sGreenXYZCoordinate.Y = sY.RGB.G * sumGreen;
            sGreenXYZCoordinate.Z = (1 - sX.RGB.G - sY.RGB.G) * sumGreen;

            sBlueXYZCoordinate.X = sX.RGB.B * sumBlue;
            sBlueXYZCoordinate.Y = sY.RGB.B * sumBlue;
            sBlueXYZCoordinate.Z = (1 - sX.RGB.B - sY.RGB.B) * sumBlue;
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="xyzCoordinate"></param>
        /// <returns></returns>
        public static FloatColor XYZCoordinateToRGB(XYZCoordinate xyzCoordinate)
        {
            FloatColor rgb = new FloatColor();

            Matrix xyzMatrix = Matrix.Invert(new Matrix(sRedXYZCoordinate.X, sGreenXYZCoordinate.X, sBlueXYZCoordinate.X, 0,
                                                        sRedXYZCoordinate.Y, sGreenXYZCoordinate.Y, sBlueXYZCoordinate.Y, 0,
                                                        sRedXYZCoordinate.Z, sGreenXYZCoordinate.Z, sBlueXYZCoordinate.Z, 0,
                                                                          0,                     0,                    0, 1));

            Matrix coordinateMatrix = new Matrix();
            coordinateMatrix.M11 = xyzCoordinate.X;
            coordinateMatrix.M21 = xyzCoordinate.Y;
            coordinateMatrix.M31 = xyzCoordinate.Z;

            Matrix rgbMatrix = xyzMatrix * coordinateMatrix;
            rgb.R = rgbMatrix.M11;
            rgb.G = rgbMatrix.M21;
            rgb.B = rgbMatrix.M31;

            return rgb;
        }

        private static readonly float DefaultLuminance = 100.0f;

        private static ChromaticityCoordinate sX;
        private static ChromaticityCoordinate sY;
        private static XYZCoordinate sWhiteXYZCoordinate;
        private static XYZCoordinate sRedXYZCoordinate;
        private static XYZCoordinate sGreenXYZCoordinate;
        private static XYZCoordinate sBlueXYZCoordinate;
    }
}
